import radicale.syno.constant as constant


class InfoItem():

    def __init__(self, value, types=[]):
        self._value = value
        self._types = [type_param.lower() for type_param in types]

        is_preferred = False
        for pref_type in constant.VCARD_PROPS_PREFER_TYPES:
            if pref_type in self._types:
                is_preferred = True
        self._is_preferred = is_preferred

    @property
    def value(self):
        return self._value

    @property
    def types(self):
        return self._types

    @property
    def is_preferred(self):
        return self._is_preferred


def get_primary_info_item_val(info_item_list):
    for info_item in info_item_list:
        if info_item.is_preferred:
            return info_item.value
    return info_item_list[0].value if len(info_item_list) > 0 else None


def get_props_items(vobject_item, props=[]):
    """
    Iterate over the items in ``vobject_item`` of ``props``.
    """
    # vobject store properties in lower case, so we transfer input props to lower case first.
    for lower_prop in (prop.lower() for prop in props):
        if lower_prop not in vobject_item.contents:
            continue
        for item in vobject_item.contents[lower_prop]:
            yield item


def get_props_first_val(vobject_item, props=[]):
    """
    For those attributes that should contain a single value, such as first_name,given_name, etc.

    We get the first value from ``vobject_item`` and return. (Although RFC may allow multiple occurrences)

    Returns None if none of the ``props`` is found.
    """
    for item in get_props_items(vobject_item, props):
        return item.value
    return None


def get_info_items(vobject_item, props=[]):
    """
    If ``vobject_item`` does not contain ``props``, an empty list is returned.

    Otherwise get the value from ``vobject_item`` and return the InfoItem list.
    """
    info_items = []
    for item in get_props_items(vobject_item, props):
        types = item.type_paramlist if hasattr(item, "type_paramlist") else []
        if isinstance(item.value, list):
            for val in item.value:
                info_items.append(InfoItem(val, types))
        else:
            info_items.append(InfoItem(item.value, types))
    return info_items
